@@ -1,6 +1,6 @@
 /* gdict-defbox.c - display widget for dictionary definitions
  *
- * Copyright (C) 2005  Emmanuele Bassi <ebassi@gmail.com>
+ * Copyright (C) 2005-2006  Emmanuele Bassi <ebassi@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,9 +33,12 @@
 
 #include "gdict-defbox.h"
 #include "gdict-utils.h"
+#include "gdict-private.h"
 #include "gdict-enum-types.h"
 #include "gdict-marshal.h"
 
+#define QUERY_MARGIN	48
+#define ERROR_MARGIN	24
 
 typedef struct
 {
@@ -44,7 +47,8 @@ typedef struct
   gint begin;
 } Definition;
 
-#define GDICT_DEFBOX_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDICT_TYPE_DEFBOX, GdictDefboxPrivate))
+#define GDICT_DEFBOX_GET_PRIVATE(obj) \
+(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDICT_TYPE_DEFBOX, GdictDefboxPrivate))
 
 struct _GdictDefboxPrivate
 {
@@ -68,6 +72,7 @@ struct _GdictDefboxPrivate
   
   gchar *word;
   gchar *database;
+  gchar *font_name;
   
   guint show_find : 1;
   guint is_searching : 1;
@@ -87,6 +92,7 @@ enum
   PROP_CONTEXT,
   PROP_WORD,
   PROP_DATABASE,
+  PROP_FONT_NAME,
   PROP_COUNT
 };
 
@@ -151,19 +157,19 @@ gdict_defbox_finalize (GObject *object)
   
   if (priv->word)
     g_free (priv->word);
+
+  if (priv->font_name)
+    g_free (priv->font_name);
   
   if (priv->definitions)
     {
-      g_slist_foreach (priv->definitions,
-      		       (GFunc) definition_free,
-      		       NULL);
+      g_slist_foreach (priv->definitions, (GFunc) definition_free, NULL);
       g_slist_free (priv->definitions);
       
       priv->definitions = NULL;
     }
   
   g_object_unref (priv->tooltips);
-  
   g_object_unref (priv->buffer);
   
   if (priv->busy_cursor)
@@ -185,7 +191,7 @@ set_gdict_context (GdictDefbox  *defbox,
     {
       if (priv->start_id)
         {
-          gdict_debug ("Removing old context handlers\n");
+          _gdict_debug ("Removing old context handlers\n");
           
           g_signal_handler_disconnect (priv->context, priv->start_id);
           g_signal_handler_disconnect (priv->context, priv->define_id);
@@ -203,7 +209,7 @@ set_gdict_context (GdictDefbox  *defbox,
           priv->error_id = 0;
         }
 
-      gdict_debug ("Removing old context\n");
+      _gdict_debug ("Removing old context\n");
       
       g_object_unref (G_OBJECT (priv->context));
     }
@@ -218,7 +224,7 @@ set_gdict_context (GdictDefbox  *defbox,
       return;
     }
 
-  gdict_debug ("Setting new context\n");
+  _gdict_debug ("Setting new context\n");
     
   priv->context = context;
   g_object_ref (G_OBJECT (priv->context));
@@ -242,6 +248,9 @@ gdict_defbox_set_property (GObject      *object,
       g_free (priv->database);
       priv->database = g_strdup (g_value_get_string (value));
       break;
+    case PROP_FONT_NAME:
+      gdict_defbox_set_font_name (defbox, g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -265,6 +274,9 @@ gdict_defbox_get_property (GObject    *object,
     case PROP_DATABASE:
       g_value_set_string (value, priv->database);
       break;
+    case PROP_FONT_NAME:
+      g_value_set_string (value, priv->font_name);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -465,21 +477,22 @@ find_entry_changed_cb (GtkWidget *widget,
   g_free (text);
 }
 
-static void
+static GtkWidget *
 create_find_pane (GdictDefbox *defbox)
 {
   GdictDefboxPrivate *priv;
+  GtkWidget *find_pane;
   GtkWidget *label;
   GtkWidget *sep;
   GtkWidget *hbox1, *hbox2;
  
   priv = defbox->priv;
   
-  priv->find_pane = gtk_hbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (priv->find_pane), 0);
+  find_pane = gtk_hbox_new (FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (find_pane), 0);
   
   hbox1 = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (priv->find_pane), hbox1, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (find_pane), hbox1, TRUE, TRUE, 0);
   gtk_widget_show (hbox1);
 
   hbox2 = gtk_hbox_new (FALSE, 12);
@@ -517,8 +530,10 @@ create_find_pane (GdictDefbox *defbox)
   
   priv->find_label = gtk_label_new (NULL);
   gtk_label_set_use_markup (GTK_LABEL (priv->find_label), TRUE);
-  gtk_box_pack_end (GTK_BOX (priv->find_pane), priv->find_label, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (find_pane), priv->find_label, FALSE, FALSE, 0);
   gtk_widget_hide (priv->find_label);
+
+  return find_pane;
 }
 
 static void
@@ -537,23 +552,44 @@ gdict_defbox_init_tags (GdictDefbox *defbox)
   gtk_text_buffer_create_tag (priv->buffer, "underline",
   			      "underline", PANGO_UNDERLINE_SINGLE,
   			      NULL);
+  
   gtk_text_buffer_create_tag (priv->buffer, "big",
   			      "scale", 1.6,
-  			      "pixels-below-lines", 5,
-  			      "pixels-above-lines", 5,
-  			      NULL);
-  gtk_text_buffer_create_tag (priv->buffer, "gray",
-  			      "foreground", "dark gray",
   			      NULL);
+  gtk_text_buffer_create_tag (priv->buffer, "small",
+		  	      "scale", PANGO_SCALE_SMALL,
+			      NULL);  
+  
   gtk_text_buffer_create_tag (priv->buffer, "emphasis",
   			      "foreground", "dark green",
   			      NULL);
+
+  gtk_text_buffer_create_tag (priv->buffer, "query-title",
+		              "left-margin", QUERY_MARGIN,
+  			      "pixels-above-lines", 5,
+  			      "pixels-below-lines", 20,
+			      NULL);
+  gtk_text_buffer_create_tag (priv->buffer, "query-from",
+		  	      "foreground", "dark gray",
+			      "scale", PANGO_SCALE_SMALL,
+		  	      "left-margin", QUERY_MARGIN,
+			      "pixels-above-lines", 5,
+			      "pixels-below-lines", 10,
+			      NULL);
+
+  gtk_text_buffer_create_tag (priv->buffer, "error-title",
+		  	      "foreground", "dark red",
+			      "left-margin", ERROR_MARGIN,
+			      NULL);
+  gtk_text_buffer_create_tag (priv->buffer, "error-message",
+		              "left-margin", ERROR_MARGIN,
+			      NULL);
 }
 
 static GObject *
-gdict_defbox_default_constructor (GType                  type,
-				  guint                  n_construct_properties,
-				  GObjectConstructParam *construct_params)
+gdict_defbox_constructor (GType                  type,
+			  guint                  n_construct_properties,
+			  GObjectConstructParam *construct_params)
 {
   GdictDefbox *defbox;
   GdictDefboxPrivate *priv;
@@ -569,6 +605,7 @@ gdict_defbox_default_constructor (GType                  type,
   gtk_widget_push_composite_child ();
   
   sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_widget_set_composite_name (sw, "gdict-defbox-scrolled-window");
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
   				  GTK_POLICY_AUTOMATIC,
   				  GTK_POLICY_AUTOMATIC);
@@ -581,12 +618,14 @@ gdict_defbox_default_constructor (GType                  type,
   gdict_defbox_init_tags (defbox);
   
   priv->text_view = gtk_text_view_new_with_buffer (priv->buffer);
+  gtk_widget_set_composite_name (priv->text_view, "gdict-defbox-text-view");
   gtk_text_view_set_editable (GTK_TEXT_VIEW (priv->text_view), FALSE);
   gtk_text_view_set_left_margin (GTK_TEXT_VIEW (priv->text_view), 4);
   gtk_container_add (GTK_CONTAINER (sw), priv->text_view);
   gtk_widget_show (priv->text_view);
   
-  create_find_pane (defbox);
+  priv->find_pane = create_find_pane (defbox);
+  gtk_widget_set_composite_name (priv->find_pane, "gdict-defbox-find-pane");
   gtk_box_pack_end (GTK_BOX (defbox), priv->find_pane, FALSE, FALSE, 0);
   
   gtk_widget_pop_composite_child ();
@@ -662,7 +701,7 @@ gdict_defbox_class_init (GdictDefboxClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   GtkBindingSet *binding_set;
   
-  gobject_class->constructor = gdict_defbox_default_constructor;
+  gobject_class->constructor = gdict_defbox_constructor;
   gobject_class->set_property = gdict_defbox_set_property;
   gobject_class->get_property = gdict_defbox_get_property;
   gobject_class->finalize = gdict_defbox_finalize;
@@ -675,13 +714,13 @@ gdict_defbox_class_init (GdictDefboxClass *klass)
    *
    * The #GdictContext object used to get the word definition.
    *
-   * Since: 1.0
+   * Since: 0.1
    */
   g_object_class_install_property (gobject_class,
   				   PROP_CONTEXT,
   				   g_param_spec_object ("context",
-  				   			_("Context"),
-  				   			_("The GdictContext object used to get the word definition"),
+  				   			"Context",
+  				   			"The GdictContext object used to get the word definition",
   				   			GDICT_TYPE_CONTEXT,
   				   			(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
   /**
@@ -690,41 +729,31 @@ gdict_defbox_class_init (GdictDefboxClass *klass)
    * The database used by the #GdictDefbox bound to this object to get the word
    * definition.
    *
-   * Since: 1.0
+   * Since: 0.1 
    */
   g_object_class_install_property (gobject_class,
 		  		   PROP_DATABASE,
 				   g_param_spec_string ("database",
-					   		_("Database"),
-							_("The database used to query the GdictContext"),
+					   		"Database",
+							"The database used to query the GdictContext",
 							GDICT_DEFAULT_DATABASE,
 							(G_PARAM_READABLE | G_PARAM_WRITABLE)));
+  /**
+   * GdictDefbox:font-name
+   *
+   * The name of the font used by the #GdictDefbox to display the definitions.
+   * use the same string you use for pango_font_description_from_string().
+   *
+   * Since: 0.3
+   */
+  g_object_class_install_property (gobject_class,
+		  		   PROP_FONT_NAME,
+				   g_param_spec_string ("font-name",
+					   		"Font Name",
+							"The font to be used by the defbox",
+							GDICT_DEFAULT_FONT_NAME,
+							(G_PARAM_READABLE | G_PARAM_WRITABLE)));
   
-  gtk_widget_class_install_style_property (widget_class,
-  					   g_param_spec_boxed ("emphasis-color",
-  					   		       _("Emphasys Color"),
-  					   		       _("Color of emphasised text"),
-  					   		       GDK_TYPE_COLOR,
-  					   		       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-  gtk_widget_class_install_style_property (widget_class,
-  					   g_param_spec_boxed ("link-color",
-  					   		       _("Link Color"),
-  					   		       _("Color of hyperlinks"),
-  					   		       GDK_TYPE_COLOR,
-  					   		       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-  gtk_widget_class_install_style_property (widget_class,
-  					   g_param_spec_boxed ("word-color",
-  					   		       _("Word Color"),
-  					   		       _("Color of the dictionary word"),
-  					   		       GDK_TYPE_COLOR,
-  					   		       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-  gtk_widget_class_install_style_property (widget_class,
-  					   g_param_spec_boxed ("source-color",
-  					   		       _("Source Color"),
-  					   		       _("Color of the dictionary source"),
-  					   		       GDK_TYPE_COLOR,
-  					   		       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-
   gdict_defbox_signals[SHOW_FIND] =
     g_signal_new ("show-find",
 		  G_OBJECT_CLASS_TYPE (gobject_class),
@@ -796,6 +825,7 @@ gdict_defbox_init (GdictDefbox *defbox)
   
   priv->context = NULL;
   priv->database = g_strdup (GDICT_DEFAULT_DATABASE);
+  priv->font_name = g_strdup (GDICT_DEFAULT_FONT_NAME);
   priv->word = NULL;
   
   priv->definitions = NULL;
@@ -815,11 +845,12 @@ gdict_defbox_init (GdictDefbox *defbox)
  *
  * Creates a new #GdictDefbox widget.  Use this widget to search for
  * a word using a #GdictContext, and to show the resulting definition(s).
- * You must set a #GdictContext for this widget using gdict_defbox_set_context().
+ * You must set a #GdictContext for this widget using
+ * gdict_defbox_set_context().
  *
  * Return value: a new #GdictDefbox widget.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 GtkWidget *
 gdict_defbox_new (void)
@@ -836,7 +867,7 @@ gdict_defbox_new (void)
  *
  * Return value: a new #GdictDefbox widget.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 GtkWidget *
 gdict_defbox_new_with_context (GdictContext *context)
@@ -856,7 +887,7 @@ gdict_defbox_new_with_context (GdictContext *context)
  * Sets @context as the #GdictContext to be used by @defbox in order
  * to retrieve the definitions of a word.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_set_context (GdictDefbox  *defbox,
@@ -876,7 +907,7 @@ gdict_defbox_set_context (GdictDefbox  *defbox,
  *
  * Return value: a #GdictContext.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 GdictContext *
 gdict_defbox_get_context (GdictDefbox *defbox)
@@ -900,7 +931,7 @@ gdict_defbox_get_context (GdictDefbox *defbox)
  * Sets @database as the database used by the #GdictContext bound to @defbox to
  * query for word definitions.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_set_database (GdictDefbox *defbox,
@@ -920,7 +951,7 @@ gdict_defbox_set_database (GdictDefbox *defbox,
  * Return value: the name of a database.  The string is owned by the
  * #GdictDefbox and should not be modified or freed.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 G_CONST_RETURN gchar *
 gdict_defbox_get_database (GdictDefbox *defbox)
@@ -941,7 +972,7 @@ gdict_defbox_get_database (GdictDefbox *defbox)
  *
  * Whether @defbox should show the find pane.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_set_show_find (GdictDefbox *defbox,
@@ -970,11 +1001,11 @@ gdict_defbox_set_show_find (GdictDefbox *defbox,
  * gdict_defbox_get_show_find:
  * @defbox: a #GdictDefbox
  *
- * FIXME
+ * Gets whether the find pane should be visible or not.
  *
- * Return value: FIXME
+ * Return value: %TRUE if the find pane is visible.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 gboolean
 gdict_defbox_get_show_find (GdictDefbox *defbox)
@@ -984,84 +1015,6 @@ gdict_defbox_get_show_find (GdictDefbox *defbox)
   return (defbox->priv->show_find == TRUE);
 }
 
-/* find the toplevel widget for @widget */
-static GtkWindow *
-get_toplevel_window (GtkWidget *widget)
-{
-  GtkWidget *toplevel;
-  
-  toplevel = gtk_widget_get_toplevel (widget);
-  if (!GTK_WIDGET_TOPLEVEL (toplevel))
-    return NULL;
-  else
-    return GTK_WINDOW (toplevel);
-}
-
-static GtkWidget *
-create_progress_dialog (GdictDefbox *defbox)
-{
-  GtkWidget *dialog;
-  GtkWidget *vbox;
-  GtkWidget *label;
-  GtkWidget *progress;
-
-  dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_transient_for (GTK_WINDOW (dialog),
-		                get_toplevel_window (GTK_WIDGET (defbox)));
-  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Progress"));
-  gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
-  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 12);
-  
-  vbox = gtk_vbox_new (FALSE, 6);
-  gtk_container_add (GTK_CONTAINER (dialog), vbox);
-  gtk_widget_show (vbox);
-
-  label = gtk_label_new (NULL);
-  g_object_set_data (G_OBJECT (dialog), "progress-label", label);
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  
-  progress = gtk_progress_bar_new ();
-  g_object_set_data (G_OBJECT (dialog), "progress-bar", progress);
-  gtk_box_pack_end (GTK_BOX (vbox), progress, FALSE, FALSE, 0);
-  gtk_widget_show (progress);
-
-  return dialog;
-}
-
-static void
-update_progress_dialog (GdictDefbox     *defbox,
-			GdictDefinition *definition)
-{
-  GdictDefboxPrivate *priv;
-  GtkWidget *progress, *label;
-  gchar *text;
-  gint total, current;
-  gdouble fraction;
-
-  priv = defbox->priv;
-
-  total = gdict_definition_get_total (definition);
-  current = g_slist_length (priv->definitions);
-  
-  text = g_strdup_printf (_("Definition for '%s' (%d/%d)"),
-			  gdict_definition_get_word (definition),
-			  current,
-			  total);
-  
-  label = g_object_get_data (G_OBJECT (priv->progress_dialog), "progress-label");
-  gtk_label_set_text (GTK_LABEL (label), text);
-  g_free (text);
-
-  fraction = ((gdouble) current / (gdouble) total);
-  progress = g_object_get_data (G_OBJECT (priv->progress_dialog), "progress-bar");
-  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
-		  		 MIN (fraction, 1.0));
-}
-
 static void
 lookup_start_cb (GdictContext *context,
 		 gpointer      user_data)
@@ -1072,8 +1025,6 @@ lookup_start_cb (GdictContext *context,
 
   priv->is_searching = TRUE;
   
-  gdict_defbox_clear (defbox);
-
   if (!priv->busy_cursor)
     priv->busy_cursor = gdk_cursor_new (GDK_WATCH);
   
@@ -1081,11 +1032,6 @@ lookup_start_cb (GdictContext *context,
   				     GTK_TEXT_WINDOW_WIDGET);
   
   gdk_window_set_cursor (window, priv->busy_cursor);
-
-  g_assert (priv->progress_dialog == NULL);
-  
-  priv->progress_dialog = create_progress_dialog (defbox);
-  gtk_widget_show_all (priv->progress_dialog);
 }
 
 static void
@@ -1103,12 +1049,6 @@ lookup_end_cb (GdictContext *context,
   gtk_text_buffer_get_start_iter (buffer, &start);
   gtk_text_buffer_place_cursor (buffer, &start);
   
-  if (priv->progress_dialog)
-    {
-      gtk_widget_destroy (priv->progress_dialog);
-      priv->progress_dialog = NULL;
-    }
-  
   window = gtk_text_view_get_window (GTK_TEXT_VIEW (priv->text_view),
   				     GTK_TEXT_WINDOW_WIDGET);
   
@@ -1116,7 +1056,7 @@ lookup_end_cb (GdictContext *context,
   
   g_free (priv->word);
   priv->word = NULL;
-  
+
   priv->is_searching = FALSE;
 }
 
@@ -1136,11 +1076,11 @@ gdict_defbox_insert_word (GdictDefbox *defbox,
   
   g_assert (GTK_IS_TEXT_BUFFER (priv->buffer));
   
-  text = g_strdup_printf ("%s\n\n", word);
+  text = g_strdup_printf ("%s\n", word);
   gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
   					    iter,
   					    text, strlen (text),
-  					    "big",
+  					    "big", "bold", "query-title",
   					    NULL);
   g_free (text);
 }
@@ -1189,11 +1129,49 @@ gdict_defbox_insert_from (GdictDefbox *defbox,
   gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
   					    iter,
   					    text, strlen (text),
-  					    "gray",
+  					    "small", "query-from",
   					    NULL);
   g_free (text);
 }
 
+static void
+gdict_defbox_insert_error (GdictDefbox *defbox,
+			   GtkTextIter *iter,
+			   const gchar *title,
+			   const gchar *message)
+{
+  GdictDefboxPrivate *priv;
+  GtkTextMark *mark;
+  GtkTextIter cur_iter;
+  
+  if (!title)
+    return;
+  
+  g_assert (GDICT_IS_DEFBOX (defbox));
+  priv = defbox->priv;
+  
+  g_assert (GTK_IS_TEXT_BUFFER (priv->buffer));
+
+  mark = gtk_text_buffer_create_mark (priv->buffer, "block-cursor", iter, FALSE);
+  gtk_text_buffer_get_iter_at_mark (priv->buffer, &cur_iter, mark);
+  
+  gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+  					    &cur_iter,
+  					    title, strlen (title),
+  					    "error-title", "big", "bold",
+  					    NULL);
+  gtk_text_buffer_get_iter_at_mark (priv->buffer, &cur_iter, mark);
+
+  gtk_text_buffer_insert (priv->buffer, &cur_iter, "\n\n", -1);
+  gtk_text_buffer_get_iter_at_mark (priv->buffer, &cur_iter, mark);
+
+  gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+		  			    &cur_iter,
+					    message, strlen (message),
+					    "error-message",
+					    NULL);
+}
+
 static void
 definition_found_cb (GdictContext    *context,
 		     GdictDefinition *definition,
@@ -1224,10 +1202,6 @@ definition_found_cb (GdictContext    *context,
   def->definition = gdict_definition_ref (definition);
   
   priv->definitions = g_slist_append (priv->definitions, def);
-
-  g_assert (priv->progress_dialog != NULL);
-  
-  update_progress_dialog (defbox, definition);
 }
 
 static void
@@ -1237,35 +1211,20 @@ error_cb (GdictContext *context,
 {
   GdictDefbox *defbox = GDICT_DEFBOX (user_data);
   GdictDefboxPrivate *priv = defbox->priv;
-
-  if (priv->progress_dialog)
-    {
-      gtk_widget_destroy (priv->progress_dialog);
-      priv->progress_dialog = NULL;
-    }
+  GtkTextIter iter;
 
   if (!error)
     return;
   
-  /* the error must not be freed */
-  gdict_show_error_dialog (GTK_WIDGET (defbox),
-  			   _("Error while looking up definition"),
-  			   error->message);
-
   gdict_defbox_clear (defbox);
+
+  gtk_text_buffer_get_start_iter (priv->buffer, &iter);
+  gdict_defbox_insert_error (defbox, &iter,
+		             _("Error while looking up definition"),
+		             error->message);
   
   g_free (priv->word);
   priv->word = NULL;
-
-  /* disconnect everything except the error callback */
-  if (priv->start_id)
-    {
-      g_signal_handler_disconnect (priv->context, priv->start_id);
-      g_signal_handler_disconnect (priv->context, priv->define_id);
-      g_signal_handler_disconnect (priv->context, priv->end_id);
-
-      priv->start_id = priv->define_id = priv->end_id = 0;
-    }
   
   defbox->priv->is_searching = FALSE;
 }
@@ -1278,7 +1237,7 @@ error_cb (GdictContext *context,
  * Searches @word inside the dictionary sources using the #GdictContext
  * provided when creating @defbox or set using gdict_defbox_set_context().
  *
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_lookup (GdictDefbox *defbox,
@@ -1293,32 +1252,37 @@ gdict_defbox_lookup (GdictDefbox *defbox,
   
   if (!priv->context)
     {
-      g_warning (_("Attempting to look up '%s' but no GdictContext defined.  Aborting."),
+      g_warning ("Attempting to look up `%s', but no GdictContext "
+		 "has been set.  Use gdict_defbox_set_context() "
+		 "before invoking gdict_defbox_lookup().",
                  word);
       return;
     }
   
   if (priv->is_searching)
     {
-      gdict_show_error_dialog (GTK_WIDGET (defbox),
-      			       _("Another search is in progress"),
-      			       _("Please wait until the current search ends."));
+      _gdict_show_error_dialog (GTK_WIDGET (defbox),
+      			        _("Another search is in progress"),
+      			        _("Please wait until the current search ends."));
       
       return;
     }
+
+  gdict_defbox_clear (defbox);
   
   if (!priv->start_id)
-    priv->start_id = g_signal_connect (priv->context, "lookup-start",
-  				       G_CALLBACK (lookup_start_cb),
+    {
+      priv->start_id = g_signal_connect (priv->context, "lookup-start",
+  				         G_CALLBACK (lookup_start_cb),
+					 defbox);
+      priv->define_id = g_signal_connect (priv->context, "definition-found",
+  				          G_CALLBACK (definition_found_cb),
+  				          defbox);
+      priv->end_id = g_signal_connect (priv->context, "lookup-end",
+  				       G_CALLBACK (lookup_end_cb),
   				       defbox);
-  if (!priv->define_id)
-    priv->define_id = g_signal_connect (priv->context, "definition-found",
-  				        G_CALLBACK (definition_found_cb),
-  				        defbox);
-  if (!priv->end_id)
-    priv->end_id = g_signal_connect (priv->context, "lookup-end",
-  				     G_CALLBACK (lookup_end_cb),
-  				     defbox);
+    }
+  
   if (!priv->error_id)
     priv->error_id = g_signal_connect (priv->context, "error",
   				       G_CALLBACK (error_cb),
@@ -1332,9 +1296,16 @@ gdict_defbox_lookup (GdictDefbox *defbox,
   			     word,
   			     &define_error);
   if (define_error)
-    gdict_show_gerror_dialog (GTK_WIDGET (defbox),
-    			      _("Error while retrieving the definition"),
-    			      define_error);
+    {
+      GtkTextIter iter;
+
+      gtk_text_buffer_get_start_iter (priv->buffer, &iter);
+      gdict_defbox_insert_error (defbox, &iter,
+    			         _("Error while retrieving the definition"),
+				 define_error->message);
+
+      g_error_free (define_error);
+    }
 }
 
 /**
@@ -1343,7 +1314,7 @@ gdict_defbox_lookup (GdictDefbox *defbox,
  *
  * Clears the buffer of @defbox
  *
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_clear (GdictDefbox *defbox)
@@ -1376,7 +1347,7 @@ gdict_defbox_clear (GdictDefbox *defbox)
  * 
  * Emits the "find-next" signal.
  * 
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_find_next (GdictDefbox *defbox)
@@ -1392,7 +1363,7 @@ gdict_defbox_find_next (GdictDefbox *defbox)
  * 
  * Emits the "find-previous" signal.
  * 
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_find_previous (GdictDefbox *defbox)
@@ -1408,7 +1379,7 @@ gdict_defbox_find_previous (GdictDefbox *defbox)
  *
  * Selects all the text displayed by @defbox
  * 
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_select_all (GdictDefbox *defbox)
@@ -1433,7 +1404,7 @@ gdict_defbox_select_all (GdictDefbox *defbox)
  *
  * Copies the selected text inside @defbox into @clipboard.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_copy_to_clipboard (GdictDefbox  *defbox,
@@ -1459,7 +1430,7 @@ gdict_defbox_copy_to_clipboard (GdictDefbox  *defbox,
  *
  * Return value: the number of definitions.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 gint
 gdict_defbox_count_definitions (GdictDefbox *defbox)
@@ -1483,7 +1454,7 @@ gdict_defbox_count_definitions (GdictDefbox *defbox)
  * Scrolls to the definition identified by @number.  If @number is -1,
  * jumps to the last definition.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 void
 gdict_defbox_jump_to_definition (GdictDefbox *defbox,
@@ -1515,7 +1486,7 @@ gdict_defbox_jump_to_definition (GdictDefbox *defbox,
       				&def_start,
       				0.0,
       				TRUE,
-      				0.5, 0.5);
+      				0.0, 0.0);
 }
 
 /**
@@ -1528,7 +1499,7 @@ gdict_defbox_jump_to_definition (GdictDefbox *defbox,
  * Return value: a newly allocated string containing the text displayed by
  *   @defbox.
  *
- * Since: 1.0
+ * Since: 0.1
  */
 gchar *
 gdict_defbox_get_text (GdictDefbox *defbox,
@@ -1553,3 +1524,62 @@ gdict_defbox_get_text (GdictDefbox *defbox,
   
   return retval;
 }
+
+/**
+ * gdict_defbox_set_font_name:
+ * @defbox: a #GdictDefbox
+ * @font_name: a font description, or %NULL
+ *
+ * Sets @font_name as the font for @defbox. It calls internally
+ * pango_font_description_from_string() and gtk_widget_modify_font().
+ * 
+ * Passing %NULL for @font_name will reset any previously set font.
+ *
+ * Since: 0.3.0
+ */
+void
+gdict_defbox_set_font_name (GdictDefbox *defbox,
+			    const gchar *font_name)
+{
+  GdictDefboxPrivate *priv;
+  PangoFontDescription *font_desc;
+
+  g_return_if_fail (GDICT_IS_DEFBOX (defbox));
+
+  priv = defbox->priv;
+
+  if (font_name)
+    {
+      font_desc = pango_font_description_from_string (font_name);
+      g_return_if_fail (font_desc != NULL);
+    }
+  else
+    font_desc = NULL;
+
+  gtk_widget_modify_font (priv->text_view, font_desc);
+
+  if (font_desc)
+    pango_font_description_free (font_desc);
+
+  g_free (priv->font_name);
+  priv->font_name = g_strdup (font_name);
+}
+
+/**
+ * gdict_defbox_get_font_name:
+ * @defbox: a #GdictDefbox
+ *
+ * Retrieves the font currently used by @defbox.
+ *
+ * Return value: a font name.  The returned string is owned by @defbox and
+ *   should not be modified or freed.
+ *
+ * Since: 0.3
+ */
+G_CONST_RETURN gchar *
+gdict_defbox_get_font_name (GdictDefbox *defbox)
+{
+  g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), NULL);
+
+  return defbox->priv->font_name;
+}
