@@ -37,17 +37,16 @@ GType logview_findbar_get_type (void);
 static gboolean
 iter_is_visible (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
-	gchar *message;
-	gboolean found = FALSE;
-	gpointer day;
 	LogviewFindBar *findbar = LOGVIEW_FINDBAR (data);
+	gboolean found = FALSE;
+	gchar *message;
+	gpointer day;
 
 	gtk_tree_model_get (model, iter, 0, &message, 1, &day, -1);
-	if (day != NULL)
+	if (day)
 		return TRUE;
-
-	if (message != NULL)
-		found = (g_strrstr (message, findbar->priv->search_string) != NULL);
+	if (message)
+		found = (g_strstr_len (message, -1, findbar->priv->search_string) != NULL);
 	return found;
 }
 
@@ -62,6 +61,7 @@ logview_findbar_clear (GtkWidget *widget, gpointer data)
  		return;
 
 	gtk_entry_set_text (GTK_ENTRY (findbar->priv->entry), "");
+	gtk_widget_hide (GTK_WIDGET (findbar));
 }
 	
 static  gboolean
@@ -70,16 +70,28 @@ logview_findbar_entry_timeout (gpointer data)
 	LogviewFindBar *findbar = LOGVIEW_FINDBAR (data);
 	LogviewWindow *logview = LOGVIEW_WINDOW (findbar->priv->logview);
 	Log *log = logview->curlog;
+	GdkCursor *cursor;
 
+	cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_window_set_cursor (GTK_WIDGET (logview)->window, cursor);
+	gdk_cursor_unref (cursor);
+	gdk_display_flush (gtk_widget_get_display (GTK_WIDGET (logview)));
+	
 	if (log->filter == NULL) {
+
 		log->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (log->model, NULL));
 		gtk_tree_model_filter_set_visible_func (log->filter, iter_is_visible, findbar, NULL);
 		gtk_tree_view_set_model (GTK_TREE_VIEW (logview->view), GTK_TREE_MODEL (log->filter));
+
 	} else {
 		gtk_tree_model_filter_refilter (log->filter);
 	}
 
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (logview->view));
+
+	gdk_window_set_cursor (GTK_WIDGET (logview)->window, NULL);
+	gdk_display_flush (gtk_widget_get_display (GTK_WIDGET (logview)));
+
 	return FALSE;
 }
 	
@@ -98,7 +110,7 @@ logview_findbar_entry_changed_cb (GtkEditable *editable,
 	if (strlen (search_string) == 0 && log->filter != NULL) {
 		g_object_unref (log->filter);
 		log->filter = NULL;
-		log_repaint (logview);
+		logview_repaint (logview);
 		return;
 	}
 
@@ -157,6 +169,7 @@ logview_findbar_finalize (GObject *object)
 	LogviewFindBar *findbar = LOGVIEW_FINDBAR (object);
 
 	g_free (findbar->priv);
+	parent_class->finalize (object);
 }
 
 static void
